@@ -1,4 +1,5 @@
 use cqrs_es::AggregateError;
+use serde_json::Error;
 use std::fmt::{Display, Formatter};
 
 /// Errors for implementations of a persistent event store.
@@ -33,3 +34,9 @@ impl From<PersistenceError> for AggregateError {
         }
     }
 }
+
+impl From<serde_json::Error> for PersistenceError {
+    fn from(err: Error) -> Self {
+        PersistenceError::UnknownError(err.to_string())
+    }
+}
