@@ -4,7 +4,9 @@ use std::marker::PhantomData;
 use async_trait::async_trait;
 use cqrs_es::{Aggregate, AggregateError, EventEnvelope, EventStore};
 
-use crate::{EventStoreAggregateContext, PersistedEventRepository};
+use crate::serialized_event::{deserialize_events, serialize_events};
+use crate::upcaster::EventUpcaster;
+use crate::{EventStoreAggregateContext, PersistedEventRepository, SerializedEvent};
 
 /// Storage engine using a database backing.
 /// This is an event-sourced `EventStore`, meaning it uses events as the
@@ -14,18 +16,19 @@ use crate::{EventStoreAggregateContext, PersistedEventRepository};
 /// see [`PersistedSnapshotStore`](struct.PersistedSnapshotStore.html).
 ///
 pub struct PersistedEventStore<R, A>
-    where
-        R: PersistedEventRepository<A>,
-        A: Aggregate + Send + Sync,
+where
+    R: PersistedEventRepository,
+    A: Aggregate + Send + Sync,
 {
     repo: R,
+    event_upcasters: Option<Vec<Box<dyn EventUpcaster>>>,
     _phantom: PhantomData<A>,
 }
 
 impl<R, A> PersistedEventStore<R, A>
-    where
-        R: PersistedEventRepository<A>,
-        A: Aggregate + Send + Sync,
+where
+    R: PersistedEventRepository,
+    A: Aggregate + Send + Sync,
 {
     /// Creates a new `PostgresStore` from the provided event repository,
     /// an `EventStore` used for configuring a new cqrs framework.
@@ -40,28 +43,36 @@ impl<R, A> PersistedEventStore<R, A>
     pub fn new(repo: R) -> Self {
         PersistedEventStore {
             repo,
+            event_upcasters: None,
             _phantom: PhantomData,
         }
     }
+
+    /// Adds configures the store to use event upcasters on load. These should be placed in the
+    /// order that they are applied.
+    ///
+    /// E.g., an upcaster for version 0.2.3 should be placed before an upcaster for version 0.2.4
+    pub fn with_upcasters(self, event_upcasters: Vec<Box<dyn EventUpcaster>>) -> Self {
+        Self {
+            repo: self.repo,
+            event_upcasters: Some(event_upcasters),
+            _phantom: Default::default(),
+        }
+    }
 }
 
 #[async_trait]
 impl<R, A> EventStore<A> for PersistedEventStore<R, A>
-    where
-        R: PersistedEventRepository<A>,
-        A: Aggregate + Send + Sync,
+where
+    R: PersistedEventRepository,
+    A: Aggregate + Send + Sync,
 {
     type AC = EventStoreAggregateContext<A>;
 
     async fn load(&self, aggregate_id: &str) -> Vec<EventEnvelope<A>> {
-        match self.repo.get_events(aggregate_id).await {
-            Ok(val) => val,
-            Err(_err) => {
-                // TODO: improved error handling
-                Default::default()
-            }
-        }
+        PersistedEventStore::load_from_repo(&self.repo, aggregate_id, &self.event_upcasters).await
     }
+
     async fn load_aggregate(&self, aggregate_id: &str) -> EventStoreAggregateContext<A> {
         let committed_events = self.load(aggregate_id).await;
         let mut aggregate = A::default();
@@ -87,26 +98,59 @@ impl<R, A> EventStore<A> for PersistedEventStore<R, A>
         let aggregate_id = context.aggregate_id.as_str();
         let current_sequence = context.current_sequence;
         let wrapped_events = self.wrap_events(aggregate_id, current_sequence, events, metadata);
-        self.repo.persist(&wrapped_events, None).await?;
+        let serialized_events: Vec<SerializedEvent> = serialize_events(&wrapped_events)?;
+        self.repo.persist::<A>(&serialized_events, None).await?;
         Ok(wrapped_events)
     }
 }
 
+impl<R, A> PersistedEventStore<R, A>
+where
+    R: PersistedEventRepository,
+    A: Aggregate + Send + Sync,
+{
+    pub(crate) async fn load_from_repo(
+        repo: &R,
+        aggregate_id: &str,
+        upcasters: &Option<Vec<Box<dyn EventUpcaster>>>,
+    ) -> Vec<EventEnvelope<A>> {
+        match repo.get_events::<A>(aggregate_id).await {
+            Ok(serialized_events) => {
+                match deserialize_events(serialized_events, upcasters) {
+                    Ok(events) => events,
+                    Err(_err) => {
+                        // TODO: improved error handling (planned for v0.3.0)
+                        Default::default()
+                    }
+                }
+            }
+            Err(_err) => {
+                // TODO: improved error handling (planned for v0.3.0)
+                Default::default()
+            }
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     use std::collections::HashMap;
+
     use cqrs_es::EventStore;
 
+    use crate::snapshot_store::test::{
+        test_serialized_event, MockRepo, TestAggregate, TestEvents, EVENT_VERSION,
+        TEST_AGGREGATE_ID,
+    };
     use crate::{EventStoreAggregateContext, PersistedEventStore, PersistenceError};
-    use crate::snapshot_store::test::{EVENT_VERSION, MockRepo, TEST_AGGREGATE_ID, test_event_envelope, TestAggregate, TestEvents};
 
     #[tokio::test]
     async fn load() {
-        let repo = MockRepo::with_events(Ok(vec![test_event_envelope(
+        let repo = MockRepo::with_events(Ok(vec![test_serialized_event(
             1,
             TestEvents::SomethingWasDone,
         )]));
-        let store = PersistedEventStore::new(repo);
+        let store = PersistedEventStore::<MockRepo, TestAggregate>::new(repo);
         let events = store.load(TEST_AGGREGATE_ID).await;
         let event = events.get(0).unwrap();
         assert_eq!(1, event.sequence);
@@ -114,11 +158,10 @@ mod test {
         assert_eq!(EVENT_VERSION, event.event_version);
     }
 
-
     #[tokio::test]
     async fn load_error() {
         let repo = MockRepo::with_events(Err(PersistenceError::OptimisticLockError));
-        let store = PersistedEventStore::new(repo);
+        let store = PersistedEventStore::<MockRepo, TestAggregate>::new(repo);
         let events = store.load(TEST_AGGREGATE_ID).await;
         assert_eq!(0, events.len())
     }
@@ -135,13 +178,10 @@ mod test {
 
     #[tokio::test]
     async fn load_aggregate_existing() {
-        let repo = MockRepo::with_events(Ok(vec![test_event_envelope(
-            1,
-            TestEvents::Started,
-        ),test_event_envelope(
-            2,
-            TestEvents::SomethingWasDone,
-        )]));
+        let repo = MockRepo::with_events(Ok(vec![
+            test_serialized_event(1, TestEvents::Started),
+            test_serialized_event(2, TestEvents::SomethingWasDone),
+        ]));
         let store = PersistedEventStore::new(repo);
         let snapshot_context = store.load_aggregate(TEST_AGGREGATE_ID).await;
         assert_eq!(2, snapshot_context.current_sequence);
@@ -159,7 +199,7 @@ mod test {
     #[should_panic]
     async fn load_aggregate_error() {
         let repo = MockRepo::with_snapshot(Err(PersistenceError::OptimisticLockError));
-        let store = PersistedEventStore::new(repo);
+        let store = PersistedEventStore::<MockRepo, TestAggregate>::new(repo);
         store.load_aggregate(TEST_AGGREGATE_ID).await;
     }
 
@@ -195,6 +235,9 @@ mod test {
         let event = event_envelopes.get(0).unwrap();
         assert_eq!(TEST_AGGREGATE_ID, event.aggregate_id);
         assert_eq!(TestEvents::Started, event.payload);
-        assert_eq!(TestEvents::SomethingWasDone, event_envelopes.get(2).unwrap().payload);
+        assert_eq!(
+            TestEvents::SomethingWasDone,
+            event_envelopes.get(2).unwrap().payload
+        );
     }
-}
\ No newline at end of file
+}
