@@ -1,10 +1,13 @@
 use std::collections::HashMap;
+use std::convert::TryInto;
 use std::marker::PhantomData;
 
 use async_trait::async_trait;
 use cqrs_es::{Aggregate, AggregateError, EventEnvelope, EventStore};
 
-use crate::{PersistedEventRepository, SnapshotStoreAggregateContext};
+use crate::serialized_event::serialize_events;
+use crate::upcaster::EventUpcaster;
+use crate::{PersistedEventRepository, PersistedEventStore, SnapshotStoreAggregateContext};
 
 /// Storage engine using a database backing.
 /// This is an snapshot-sourced `EventStore`, meaning it uses the serialized aggregate as the
@@ -16,16 +19,17 @@ use crate::{PersistedEventRepository, SnapshotStoreAggregateContext};
 ///
 pub struct PersistedSnapshotStore<R, A>
 where
-    R: PersistedEventRepository<A>,
+    R: PersistedEventRepository,
     A: Aggregate + Send + Sync,
 {
     repo: R,
+    event_upcasters: Option<Vec<Box<dyn EventUpcaster>>>,
     _phantom: PhantomData<A>,
 }
 
 impl<R, A> PersistedSnapshotStore<R, A>
 where
-    R: PersistedEventRepository<A>,
+    R: PersistedEventRepository,
     A: Aggregate + Send + Sync,
 {
     /// Creates a new `PostgresSnapshotStore` from the provided database connection pool,
@@ -40,6 +44,7 @@ where
     pub fn new(snapshot_repo: R) -> Self {
         PersistedSnapshotStore {
             repo: snapshot_repo,
+            event_upcasters: None,
             _phantom: PhantomData,
         }
     }
@@ -48,32 +53,25 @@ where
 #[async_trait]
 impl<R, A> EventStore<A> for PersistedSnapshotStore<R, A>
 where
-    R: PersistedEventRepository<A>,
+    R: PersistedEventRepository,
     A: Aggregate + Send + Sync,
 {
     type AC = SnapshotStoreAggregateContext<A>;
 
     async fn load(&self, aggregate_id: &str) -> Vec<EventEnvelope<A>> {
-        match self.repo.get_events(aggregate_id).await {
-            Ok(val) => val,
-            Err(_err) => {
-                // TODO: improved error handling
-                Default::default()
-            }
-        }
+        PersistedEventStore::load_from_repo(&self.repo, aggregate_id, &self.event_upcasters).await
     }
     async fn load_aggregate(&self, aggregate_id: &str) -> SnapshotStoreAggregateContext<A> {
-        match self.repo.get_snapshot(aggregate_id).await {
+        match self.repo.get_snapshot::<A>(aggregate_id).await {
             Ok(snapshot) => match snapshot {
-                Some(snapshot) => snapshot,
-                None => SnapshotStoreAggregateContext {
-                    aggregate_id: aggregate_id.to_string(),
-                    aggregate: Default::default(),
-                    current_sequence: 0,
-                    current_snapshot: 0,
-                },
+                Some(snapshot) => {
+                    // TODO: improved error handling (planned for v0.3.0)
+                    snapshot.try_into().expect("snapshot can be deserialized")
+                }
+                None => SnapshotStoreAggregateContext::new(aggregate_id),
             },
             Err(e) => {
+                // TODO: improved error handling (planned for v0.3.0)
                 panic!("{}", e);
             }
         }
@@ -90,13 +88,13 @@ where
         }
         let aggregate_id = context.aggregate_id.clone();
         let next_snapshot = context.current_snapshot + 1;
-        let wrapped_events =
-            self.wrap_events(&aggregate_id, context.current_sequence, events, metadata);
+        let seq = context.current_sequence;
+        let wrapped_events = self.wrap_events(&aggregate_id, seq, events, metadata);
+        let serialized_events = serialize_events(&wrapped_events)?;
+        let payload = serde_json::to_value(context.aggregate)?;
+        let snapshot_update = Some((aggregate_id, payload, next_snapshot));
         self.repo
-            .persist(
-                &wrapped_events,
-                Some((aggregate_id, context.aggregate, next_snapshot)),
-            )
+            .persist::<A>(&serialized_events, snapshot_update)
             .await?;
         Ok(wrapped_events)
     }
@@ -108,12 +106,14 @@ pub(crate) mod test {
     use std::sync::Mutex;
 
     use async_trait::async_trait;
-    use cqrs_es::{Aggregate, AggregateError, DomainEvent, EventEnvelope, EventStore};
+    use cqrs_es::{Aggregate, AggregateError, DomainEvent, EventStore};
     use serde::{Deserialize, Serialize};
+    use serde_json::json;
+    use serde_json::Value;
 
     use crate::{
-        PersistedEventRepository, PersistedSnapshotStore, PersistenceError,
-        SnapshotStoreAggregateContext,
+        PersistedEventRepository, PersistedSnapshotStore, PersistenceError, SerializedEvent,
+        SerializedSnapshot, SnapshotStoreAggregateContext,
     };
 
     #[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
@@ -169,26 +169,15 @@ pub(crate) mod test {
     }
 
     pub(crate) struct MockRepo {
-        events_result: Mutex<Option<Result<Vec<EventEnvelope<TestAggregate>>, PersistenceError>>>,
-        snapshot_result: Mutex<
-            Option<Result<Option<SnapshotStoreAggregateContext<TestAggregate>>, PersistenceError>>,
-        >,
+        events_result: Mutex<Option<Result<Vec<SerializedEvent>, PersistenceError>>>,
+        snapshot_result: Mutex<Option<Result<Option<SerializedSnapshot>, PersistenceError>>>,
         persist_check: Mutex<
-            Option<
-                Box<
-                    dyn FnOnce(
-                            &[EventEnvelope<TestAggregate>],
-                            Option<(String, TestAggregate, usize)>,
-                        ) + Send,
-                >,
-            >,
+            Option<Box<dyn FnOnce(&[SerializedEvent], Option<(String, Value, usize)>) + Send>>,
         >,
     }
 
     impl MockRepo {
-        pub(crate) fn with_events(
-            result: Result<Vec<EventEnvelope<TestAggregate>>, PersistenceError>,
-        ) -> Self {
+        pub(crate) fn with_events(result: Result<Vec<SerializedEvent>, PersistenceError>) -> Self {
             Self {
                 events_result: Mutex::new(Some(result)),
                 snapshot_result: Mutex::new(None),
@@ -196,7 +185,7 @@ pub(crate) mod test {
             }
         }
         pub(crate) fn with_snapshot(
-            result: Result<Option<SnapshotStoreAggregateContext<TestAggregate>>, PersistenceError>,
+            result: Result<Option<SerializedSnapshot>, PersistenceError>,
         ) -> Self {
             Self {
                 events_result: Mutex::new(None),
@@ -206,8 +195,7 @@ pub(crate) mod test {
         }
         pub(crate) fn with_commit(
             test_function: Box<
-                dyn FnOnce(&[EventEnvelope<TestAggregate>], Option<(String, TestAggregate, usize)>)
-                    + Send,
+                dyn FnOnce(&[SerializedEvent], Option<(String, Value, usize)>) + Send,
             >,
         ) -> Self {
             Self {
@@ -219,27 +207,30 @@ pub(crate) mod test {
     }
 
     #[async_trait]
-    impl PersistedEventRepository<TestAggregate> for MockRepo {
-        async fn get_events(
+    impl PersistedEventRepository for MockRepo {
+        async fn get_events<A: Aggregate>(
             &self,
             _aggregate_id: &str,
-        ) -> Result<Vec<EventEnvelope<TestAggregate>>, PersistenceError> {
+        ) -> Result<Vec<SerializedEvent>, PersistenceError> {
             self.events_result.lock().unwrap().take().unwrap()
         }
-        async fn get_last_events(&self, _aggregate_id: &str, _number_events: usize) -> Result<Vec<EventEnvelope<TestAggregate>>, PersistenceError> {
-            todo!()
-        }
-        async fn get_snapshot(
+        async fn get_last_events<A: Aggregate>(
             &self,
             _aggregate_id: &str,
-        ) -> Result<Option<SnapshotStoreAggregateContext<TestAggregate>>, PersistenceError>
-        {
+            _number_events: usize,
+        ) -> Result<Vec<SerializedEvent>, PersistenceError> {
+            todo!()
+        }
+        async fn get_snapshot<A: Aggregate>(
+            &self,
+            _aggregate_id: &str,
+        ) -> Result<Option<SerializedSnapshot>, PersistenceError> {
             self.snapshot_result.lock().unwrap().take().unwrap()
         }
-        async fn persist(
+        async fn persist<A: Aggregate>(
             &self,
-            events: &[EventEnvelope<TestAggregate>],
-            snapshot_update: Option<(String, TestAggregate, usize)>,
+            events: &[SerializedEvent],
+            snapshot_update: Option<(String, Value, usize)>,
         ) -> Result<(), PersistenceError> {
             let test = self.persist_check.lock().unwrap().take().unwrap();
             test(events, snapshot_update);
@@ -252,11 +243,11 @@ pub(crate) mod test {
 
     #[tokio::test]
     async fn load() {
-        let repo = MockRepo::with_events(Ok(vec![test_event_envelope(
+        let repo = MockRepo::with_events(Ok(vec![test_serialized_event(
             1,
             TestEvents::SomethingWasDone,
         )]));
-        let store = PersistedSnapshotStore::new(repo);
+        let store = PersistedSnapshotStore::<MockRepo, TestAggregate>::new(repo);
         let events = store.load(TEST_AGGREGATE_ID).await;
         let event = events.get(0).unwrap();
         assert_eq!(1, event.sequence);
@@ -267,7 +258,7 @@ pub(crate) mod test {
     #[tokio::test]
     async fn load_error() {
         let repo = MockRepo::with_events(Err(PersistenceError::OptimisticLockError));
-        let store = PersistedSnapshotStore::new(repo);
+        let store = PersistedSnapshotStore::<MockRepo, TestAggregate>::new(repo);
         let events = store.load(TEST_AGGREGATE_ID).await;
         assert_eq!(0, events.len())
     }
@@ -285,15 +276,15 @@ pub(crate) mod test {
 
     #[tokio::test]
     async fn load_aggregate_existing() {
-        let repo =
-            MockRepo::with_snapshot(Ok(Some(SnapshotStoreAggregateContext::<TestAggregate> {
-                aggregate_id: TEST_AGGREGATE_ID.to_string(),
-                aggregate: TestAggregate {
-                    something_happened: 3,
-                },
-                current_sequence: 3,
-                current_snapshot: 2,
-            })));
+        let repo = MockRepo::with_snapshot(Ok(Some(SerializedSnapshot {
+            aggregate_id: TEST_AGGREGATE_ID.to_string(),
+            aggregate: serde_json::to_value(TestAggregate {
+                something_happened: 3,
+            })
+            .unwrap(),
+            current_sequence: 3,
+            current_snapshot: 2,
+        })));
         let store = PersistedSnapshotStore::new(repo);
         let snapshot_context = store.load_aggregate(TEST_AGGREGATE_ID).await;
         assert_eq!(2, snapshot_context.current_snapshot);
@@ -312,7 +303,7 @@ pub(crate) mod test {
     #[should_panic]
     async fn load_aggregate_error() {
         let repo = MockRepo::with_snapshot(Err(PersistenceError::OptimisticLockError));
-        let store = PersistedSnapshotStore::new(repo);
+        let store = PersistedSnapshotStore::<MockRepo, TestAggregate>::new(repo);
         store.load_aggregate(TEST_AGGREGATE_ID).await;
     }
 
@@ -331,9 +322,9 @@ pub(crate) mod test {
             assert_eq!(TEST_AGGREGATE_ID, aggregate_id.as_str());
             assert_eq!(1, snapshot_version);
             assert_eq!(
-                TestAggregate {
+                json!(TestAggregate {
                     something_happened: 2
-                },
+                }),
                 aggregate
             );
         }));
@@ -360,15 +351,24 @@ pub(crate) mod test {
         let event = event_envelopes.get(0).unwrap();
         assert_eq!(TEST_AGGREGATE_ID, event.aggregate_id);
         assert_eq!(TestEvents::Started, event.payload);
-        assert_eq!(TestEvents::SomethingWasDone, event_envelopes.get(2).unwrap().payload);
+        assert_eq!(
+            TestEvents::SomethingWasDone,
+            event_envelopes.get(2).unwrap().payload
+        );
     }
 
-    pub(crate) fn test_event_envelope(seq: usize, payload: TestEvents) -> EventEnvelope<TestAggregate> {
-        EventEnvelope::<TestAggregate>::new(
+    pub(crate) fn test_serialized_event(seq: usize, event: TestEvents) -> SerializedEvent {
+        let event_type = event.event_type().to_string();
+        let event_version = event.event_version().to_string();
+        let payload = serde_json::to_value(&event).unwrap();
+        SerializedEvent::new(
             TEST_AGGREGATE_ID.to_string(),
             seq,
-            TestAggregate::aggregate_type().to_string(),
+            "TestAggregate".to_string(),
+            event_type,
+            event_version,
             payload,
+            serde_json::to_value(HashMap::<String, String>::new()).unwrap(),
         )
     }
 }
